@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
 ## system-config-printer
 
@@ -22,14 +22,16 @@
 
 import cups
 from .cupshelpers import parseDeviceID
-import xmldriverprefs
+from . import xmldriverprefs
 import itertools
 import string
 import time
 import locale
 import os.path
+import functools
 import re
 from . import _debugprint, set_debugprint_fn
+from functools import reduce
 
 __all__ = ['ppdMakeModelSplit',
            'PPDs']
@@ -92,13 +94,13 @@ _HP_MODEL_BY_NAME = {
 }
 
 _RE_turboprint = re.compile ("turboprint")
-_RE_version_numbers = re.compile (" v(?:er\.)?\d(?:\d*\.\d+)?(?: |$)")
+_RE_version_numbers = re.compile (r" v(?:er\.)?\d(?:\d*\.\d+)?(?: |$)")
 _RE_ignore_suffix = re.compile (","
                                 "| hpijs"
                                 "| foomatic/"
                                 "| - "
                                 "| w/"
-                                "| \("
+                                "| \\("
                                 "| postscript"
                                 "| ps"
                                 "| pdf"
@@ -137,7 +139,7 @@ def ppdMakeModelSplit (ppd_make_and_model):
             break
 
     # Handle PPDs provided by Turboprint
-    if make == None and _RE_turboprint.search (l):
+    if make is None and _RE_turboprint.search (l):
         t = ppd_make_and_model.find (" TurboPrint")
         if t != -1:
             t2 = ppd_make_and_model.rfind (" TurboPrint")
@@ -237,7 +239,7 @@ def ppdMakeModelSplit (ppd_make_and_model):
         modell = model.lower ()
 
     if makel == "hp":
-        for (name, fullname) in _HP_MODEL_BY_NAME.iteritems ():
+        for name, fullname in _HP_MODEL_BY_NAME.items ():
             if modell.startswith (name):
                 model = fullname + model[len (name):]
                 modell = model.lower ()
@@ -342,10 +344,10 @@ class PPDs:
 
         self.drivertypes = xmldriverprefs.DriverTypes ()
         self.preforder = xmldriverprefs.PreferenceOrder ()
-        if xml_dir == None:
+        if xml_dir is None:
             xml_dir = os.environ.get ("CUPSHELPERS_XMLDIR")
-            if xml_dir == None:
-                import config
+            if xml_dir is None:
+                from . import config
                 xml_dir = os.path.join (config.sysconfdir, "cupshelpers")
 
         try:
@@ -355,11 +357,11 @@ class PPDs:
             self.drivertypes.load (drivertypes)
             self.preforder.load (preferenceorder)
         except Exception as e:
-            print "Error loading %s: %s" % (xmlfile, e)
+            print("Error loading %s: %s" % (xmlfile, e))
             self.drivertypes = None
             self.preforder = None
 
-        if (language == None or
+        if (language is None or
             language == "C" or
             language == "POSIX"):
             language = "en_US"
@@ -371,7 +373,7 @@ class PPDs:
             short_language = language
 
         to_remove = []
-        for ppdname, ppddict in self.ppds.iteritems ():
+        for ppdname, ppddict in self.ppds.items ():
             try:
                 natural_language = _singleton (ppddict['ppd-natural-language'])
             except KeyError:
@@ -396,7 +398,7 @@ class PPDs:
         # CUPS sets the 'raw' model's ppd-make-and-model to 'Raw Queue'
         # which unfortunately then appears as manufacturer Raw and
         # model Queue.  Use 'Generic' for this model.
-        if self.ppds.has_key ('raw'):
+        if 'raw' in self.ppds:
             makemodel = _singleton (self.ppds['raw']['ppd-make-and-model'])
             if not makemodel.startswith ("Generic "):
                 self.ppds['raw']['ppd-make-and-model'] = "Generic " + makemodel
@@ -407,8 +409,8 @@ class PPDs:
         to the current locale
 	"""
         self._init_makes ()
-        makes_list = self.makes.keys ()
-        makes_list.sort (locale.strcoll)
+        makes_list = list(self.makes.keys ())
+        makes_list.sort (key=locale.strxfrm)
         try:
             # "Generic" should be listed first.
             makes_list.remove ("Generic")
@@ -424,10 +426,15 @@ class PPDs:
 	"""
         self._init_makes ()
         try:
-            models_list = self.makes[make].keys ()
+            models_list = list(self.makes[make].keys ())
         except KeyError:
             return []
-        models_list.sort (key=lambda x: normalize (x), cmp=cups.modelSort)
+
+        def compare_models (a,b):
+            first = normalize (a)
+            second = normalize (b)
+            return cups.modelSort(first, second)
+        models_list.sort(key=functools.cmp_to_key(compare_models))
         return models_list
 
     def getInfoFromModel (self, make, model):
@@ -454,8 +461,8 @@ class PPDs:
     def getStatusFromFit (self, fit):
         return self._fit_to_status.get (fit, xmldriverprefs.DriverType.FIT_NONE)
 
-    def orderPPDNamesByPreference (self, ppdnamelist=[],
-                                   downloadedfiles=[],
+    def orderPPDNamesByPreference (self, ppdnamelist=None,
+                                   downloadedfiles=None,
                                    make_and_model=None,
                                    devid=None, fit=None):
         """
@@ -475,7 +482,13 @@ class PPDs:
         @type fit: dict of PPD name:fit
 	@returns: string list
 	"""
-        if fit == None:
+        if ppdnamelist is None:
+            ppdnamelist = []
+
+        if downloadedfiles is None:
+            downloadedfiles = []
+
+        if fit is None:
             fit = {}
 
         if self.drivertypes and self.preforder:
@@ -490,7 +503,7 @@ class PPDs:
             orderedppds = self.drivertypes.get_ordered_ppdnames (orderedtypes,
                                                                  ppds, fit)
             _debugprint("PPDs with assigned driver types in priority order: %s" % repr(orderedppds))
-            ppdnamelist = map (lambda (typ, name): name, orderedppds)
+            ppdnamelist = [typ_name[1] for typ_name in orderedppds]
             _debugprint("Resulting PPD list in priority order: %s" % repr(ppdnamelist))
 
         # Special handling for files we've downloaded.  First collect
@@ -519,7 +532,7 @@ class PPDs:
         return ppdnamelist
 
     def getPPDNamesFromDeviceID (self, mfg, mdl, description="",
-                                 commandsets=[], uri=None,
+                                 commandsets=None, uri=None,
                                  make_and_model=None):
         """
 	Obtain a best-effort PPD match for an IEEE 1284 Device ID.
@@ -543,6 +556,9 @@ class PPDs:
         orig_mdl = mdl
         self._init_ids ()
 
+        if commandsets is None:
+            commandsets = []
+
         # Start with an empty result list and build it up using
         # several search methods, in increasing order of fuzziness.
         fit = {}
@@ -566,10 +582,10 @@ class PPDs:
             try:
                 for each in self.ids["hp"][mdll]:
                     fit[each] = self.FIT_EXACT
-                print ("**** Incorrect IEEE 1284 Device ID: %s" %
-                       self.ids["hp"][mdll])
-                print "**** Actual ID is MFG:%s;MDL:%s;" % (mfg, mdl)
-                print "**** Please report a bug against the HPLIP component"
+                _debugprint ("**** Incorrect IEEE 1284 Device ID: %s" %
+                             self.ids["hp"][mdll])
+                _debugprint ("**** Actual ID is MFG:%s;MDL:%s;" % (mfg, mdl))
+                _debugprint ("**** Please report a bug against the HPLIP component")
                 id_matched = True
             except KeyError:
                 pass
@@ -589,12 +605,12 @@ class PPDs:
         mfgrepl = {"hewlett-packard": "hp",
                    "lexmark international": "lexmark",
                    "kyocera": "kyocera mita"}
-        if self.lmakes.has_key (mfgl):
+        if mfgl in self.lmakes:
             # Found manufacturer.
             make = self.lmakes[mfgl]
-        elif mfgrepl.has_key (mfgl):
+        elif mfgl in mfgrepl:
             rmfg = mfgrepl[mfgl]
-            if self.lmakes.has_key (rmfg):
+            if rmfg in self.lmakes:
                 mfg = rmfg
                 mfgl = mfg
                 # Found manufacturer (after mapping to canonical name)
@@ -602,7 +618,7 @@ class PPDs:
                 make = self.lmakes[mfgl]
 
         _debugprint ("make: %s" % make)
-        if make != None:
+        if make is not None:
             mdls = self.makes[make]
             mdlsl = self.lmodels[normalize(make)]
 
@@ -613,7 +629,7 @@ class PPDs:
                     mdll = normalize (mdl)
                     _debugprint ("unprefixed mdll: %s" % mdll)
 
-            if self.lmodels[mfgl].has_key (mdll):
+            if mdll in self.lmodels[mfgl]:
                 model = mdlsl[mdll]
                 for each in mdls[model].keys ():
                     fit[each] = self.FIT_EXACT
@@ -624,9 +640,9 @@ class PPDs:
                 (mfg2, mdl2) = ppdMakeModelSplit (mfg + " " + mdl)
                 mdl2l = normalize (mdl2)
                 _debugprint ("re-split mdll: %s" % mdl2l)
-                if self.lmodels[mfgl].has_key (mdl2l):
+                if mdl2l in self.lmodels[mfgl]:
                     model = mdlsl[mdl2l]
-                    for each in mdls[model].keys ():
+                    for each in list(mdls[model].keys ()):
                         fit[each] = self.FIT_EXACT
                         _debugprint ("%s: %s" % (fit[each], each))
       
@@ -648,6 +664,24 @@ class PPDs:
                     fit[driver] = self.FIT_GENERIC
                     _debugprint ("%s: %s" % (fit[driver], driver))
 
+        # Check by the URI whether our printer is connected via IPP
+        # and if not, remove the PPD entries for driverless printing
+        # (ppdname = "driverless:..." from the list)
+        if (not uri or
+            (not uri.startswith("ipp:") and
+             not uri.startswith("ipps:") and
+             (not uri.startswith("dnssd") or
+              not "._ipp" in uri))):
+            failed = set()
+            for ppdname in fit.keys ():
+                if (ppdname.startswith("driverless:")):
+                    failed.add (ppdname)
+            if (len(failed) > 0):
+                _debugprint ("Removed %s due to non-IPP connection" % failed)
+                for each in failed:
+                    del fit[each]
+            failed = set()
+
         # What about the CMD field of the Device ID?  Some devices
         # have optional units for page description languages, such as
         # PostScript, and they will report different CMD strings
@@ -746,7 +780,7 @@ class PPDs:
 
             if not found:
                 _debugprint ("No fallback available; choosing any")
-                fit[self.ppds.keys ()[0]] = self.FIT_NONE
+                fit[list(self.ppds.keys ())[0]] = self.FIT_NONE
 
         if not id_matched:
             sanitised_uri = re.sub (pattern="//[^@]*@/?", repl="//",
@@ -761,14 +795,14 @@ class PPDs:
             if description:
                 id += "DES:%s;" % description
 
-            print "No ID match for device %s:" % sanitised_uri
-            print id
+            _debugprint ("No ID match for device %s:" % sanitised_uri)
+            _debugprint (id)
 
         return fit
 
     def getPPDNameFromDeviceID (self, mfg, mdl, description="",
-                                commandsets=[], uri=None,
-                                downloadedfiles=[],
+                                commandsets=None, uri=None,
+                                downloadedfiles=None,
                                 make_and_model=None):
         """
 	Obtain a best-effort PPD match for an IEEE 1284 Device ID.
@@ -804,6 +838,12 @@ class PPDs:
 	@returns: an integer,string pair of (status,ppd-name)
 	"""
 
+        if commandsets is None:
+            commandsets = []
+
+        if downloadedfiles is None:
+            downloadedfiles = []
+
         fit = self.getPPDNamesFromDeviceID (mfg, mdl, description,
                                             commandsets, uri,
                                             make_and_model)
@@ -815,14 +855,14 @@ class PPDs:
         devid = { "MFG": mfg, "MDL": mdl,
                   "DES": description,
                   "CMD": commandsets }
-        ppdnamelist = self.orderPPDNamesByPreference (fit.keys (),
+        ppdnamelist = self.orderPPDNamesByPreference (list(fit.keys ()),
                                                       downloadedfiles,
                                                       make_and_model,
                                                       devid, fit)
         _debugprint ("Found PPDs: %s" % str (ppdnamelist))
 
         status = self.getStatusFromFit (fit[ppdnamelist[0]])
-        print "Using %s (status: %d)" % (ppdnamelist[0], status)
+        _debugprint ("Using %s (status: %d)" % (ppdnamelist[0], status))
         return (status, ppdnamelist[0])
 
     def _findBestMatchPPDs (self, mdls, mdl):
@@ -839,12 +879,12 @@ class PPDs:
             mdl = mdl[:-7]
         best_mdl = None
         best_matchlen = 0
-        mdlnames = mdls.keys ()
+        mdlnames = list(mdls.keys ())
 
         # Perform a case-insensitive model sort on the names.
-        mdlnamesl = map (lambda x: (x, x.lower()), mdlnames)
+        mdlnamesl = [(x, x.lower()) for x in mdlnames]
         mdlnamesl.append ((mdl, mdll))
-        mdlnamesl.sort (lambda x, y: cups.modelSort(x[1], y[1]))
+        mdlnamesl.sort (key=functools.cmp_to_key(lambda x, y: cups.modelSort(x[1], y[1])))
         i = mdlnamesl.index ((mdl, mdll))
         candidates = [mdlnamesl[i - 1]]
         if i + 1 < len (mdlnamesl):
@@ -859,7 +899,7 @@ class PPDs:
         for (candidate, candidatel) in candidates:
             prefix = os.path.commonprefix ([candidatel, mdll])
             if len (prefix) > best_matchlen:
-                best_mdl = mdls[candidate].keys ()
+                best_mdl = list(mdls[candidate].keys ())
                 best_matchlen = len (prefix)
                 _debugprint ("%s: match length %d" % (candidate, best_matchlen))
 
@@ -878,11 +918,11 @@ class PPDs:
             # field and look for a match based solely on that.  If
             # there are digits, try lowering the number of
             # significant figures.
-            mdlnames.sort (cups.modelSort)
-            mdlitems = map (lambda x: (x.lower (), mdls[x]), mdlnames)
+            mdlnames.sort (key=functools.cmp_to_key(cups.modelSort))
+            mdlitems = [(x.lower (), mdls[x]) for x in mdlnames]
             modelid = None
             for word in mdll.split (' '):
-                if modelid == None:
+                if modelid is None:
                     modelid = word
 
                 have_digits = False
@@ -930,7 +970,7 @@ class PPDs:
                                 break
 
                         if found:
-                            best_mdl = ppds.keys ()
+                            best_mdl = list(ppds.keys ())
                             break
 
                     if found:
@@ -946,9 +986,13 @@ class PPDs:
 
         return (fit, ppdnamelist)
 
-    def _getPPDNameFromCommandSet (self, commandsets=[]):
+    def _getPPDNameFromCommandSet (self, commandsets=None):
         """Return ppd-name list or None, given a list of strings representing
         the command sets supported."""
+
+        if commandsets is None:
+            commandsets = []
+
         try:
             self._init_makes ()
             models = self.makes["Generic"]
@@ -962,7 +1006,7 @@ class PPDs:
                     return ppds
             return None
 
-        cmdsets = map (lambda x: x.lower (), commandsets)
+        cmdsets = [x.lower () for x in commandsets]
         if (("postscript" in cmdsets) or ("postscript2" in cmdsets) or
             ("postscript level 2 emulation" in cmdsets)):
             return get ("PostScript")
@@ -991,7 +1035,7 @@ class PPDs:
         lmakes = {}
         lmodels = {}
         aliases = {} # Generic model name: set(specific model names)
-        for ppdname, ppddict in self.ppds.iteritems ():
+        for ppdname, ppddict in self.ppds.items ():
             # One entry for ppd-make-and-model
             ppd_make_and_model = _singleton (ppddict['ppd-make-and-model'])
             ppd_mm_split = ppdMakeModelSplit (ppd_make_and_model)
@@ -1008,8 +1052,7 @@ class PPDs:
             ppd_products = ppddict.get ('ppd-product', [])
             if not isinstance (ppd_products, list):
                 ppd_products = [ppd_products]
-            ppd_products = set (filter (lambda x: x.startswith ("("),
-                                        ppd_products))
+            ppd_products = set ([x for x in ppd_products if x.startswith ("(")])
             if ppd_products:
                 # If there is only one ppd-product value it is
                 # unlikely to be useful.
@@ -1040,14 +1083,14 @@ class PPDs:
             for make, model in ppd_makes_and_models:
                 lmake = normalize (make)
                 lmodel = normalize (model)
-                if not lmakes.has_key (lmake):
+                if lmake not in lmakes:
                     lmakes[lmake] = make
                     lmodels[lmake] = {}
                     makes[make] = {}
                 else:
                     make = lmakes[lmake]
 
-                if not lmodels[lmake].has_key (lmodel):
+                if lmodel not in lmodels[lmake]:
                     lmodels[lmake][lmodel] = model
                     makes[make][model] = {}
                 else:
@@ -1061,22 +1104,21 @@ class PPDs:
 
             if ppd_makes_and_models:
                 (make, model) = ppd_mm_split
-                if aliases.has_key (make):
+                if make in aliases:
                     models = aliases[make].get (model, set())
                 else:
                     aliases[make] = {}
                     models = set()
 
-                models = models.union (map (lambda x: x[1],
-                                            ppd_makes_and_models))
+                models = models.union ([x[1] for x in ppd_makes_and_models])
                 aliases[make][model] = models
 
         # Now, for each set of model aliases, add all drivers from the
         # "main" (generic) model name to each of the specific models.
-        for make, models in aliases.iteritems ():
+        for make, models in aliases.items ():
             lmake = normalize (make)
             main_make = lmakes[lmake]
-            for model, modelnames in models.iteritems ():
+            for model, modelnames in models.items ():
                 main_model = lmodels[lmake].get (normalize (model))
                 if not main_model:
                     continue
@@ -1098,7 +1140,7 @@ class PPDs:
             return
 
         ids = {}
-        for ppdname, ppddict in self.ppds.iteritems ():
+        for ppdname, ppddict in self.ppds.items ():
             id = _singleton (ppddict.get ('ppd-device-id'))
             if not id:
                 continue
@@ -1115,10 +1157,10 @@ class PPDs:
             if bad:
                 continue
 
-            if not ids.has_key (lmfg):
+            if lmfg not in ids:
                 ids[lmfg] = {}
 
-            if not ids[lmfg].has_key (lmdl):
+            if lmdl not in ids[lmfg]:
                 ids[lmfg][lmdl] = []
 
             ids[lmfg][lmdl].append (ppdname)
@@ -1126,191 +1168,4 @@ class PPDs:
         self.ids = ids
 
 def _show_help():
-    print "usage: ppds.py [--deviceid] [--list-models] [--list-ids] [--debug]"
-
-def _self_test(argv):
-    import sys, getopt
-    try:
-        opts, args = getopt.gnu_getopt (argv[1:], '',
-                                        ['help',
-                                         'deviceid',
-                                         'list-models',
-                                         'list-ids',
-                                         'debug'])
-    except getopt.GetoptError:
-        _show_help()
-        sys.exit (1)
-
-    stdin_deviceid = False
-    list_models = False
-    list_ids = False
-
-    for opt, optarg in opts:
-        if opt == "--help":
-            _show_help ()
-            sys.exit (0)
-        if opt == "--deviceid":
-            stdin_deviceid = True
-        elif opt == "--list-models":
-            list_models = True
-        elif opt == "--list-ids":
-            list_ids = True
-        elif opt == "--debug":
-            def _dprint(x):
-                try:
-                    print x
-                except:
-                    pass
-
-            set_debugprint_fn (_dprint)
-
-    picklefile="pickled-ppds"
-    import pickle
-    try:
-        f = open (picklefile, "r")
-        cupsppds = pickle.load (f)
-    except IOError:
-        f = open (picklefile, "w")
-        c = cups.Connection ()
-        try:
-            cupsppds = c.getPPDs2 ()
-            print "Using getPPDs2()"
-        except AttributeError:
-            # Need pycups >= 1.9.52 for getPPDs2
-            cupsppds = c.getPPDs ()
-            print "Using getPPDs()"
-
-        pickle.dump (cupsppds, f)
-
-    xml_dir = os.environ.get ("top_srcdir")
-    if xml_dir:
-        xml_dir = os.path.join (xml_dir, "xml")
-
-    ppds = PPDs (cupsppds, xml_dir=xml_dir)
-    makes = ppds.getMakes ()
-    models_count = 0
-    for make in makes:
-        models = ppds.getModels (make)
-        models_count += len (models)
-        if list_models:
-            print make
-            for model in models:
-                print "  " + model
-    print "%d makes, %d models" % (len (makes), models_count)
-    ppds.getPPDNameFromDeviceID ("HP", "PSC 2200 Series")
-    makes = ppds.ids.keys ()
-    models_count = 0
-    for make in makes:
-        models = ppds.ids[make]
-        models_count += len (models)
-        if list_ids:
-            print make
-            for model in models:
-                print "  %s (%d)" % (model, len (ppds.ids[make][model]))
-                for driver in ppds.ids[make][model]:
-                    print "    " + driver
-    print "%d ID makes, %d ID models" % (len (makes), models_count)
-
-    print "\nID matching tests\n"
-
-    MASK_STATUS = (1 << 2) - 1
-    FLAG_INVERT = (1 << 2)
-    FLAG_IGNORE_STATUS = (1 << 3)
-    idlist = [
-        # Format is:
-        # (ID string, max status code (plus flags),
-        #  expected ppd-make-and-model RE match)
-
-        # Specific models
-        ("MFG:EPSON;CMD:ESCPL2,BDC,D4,D4PX;MDL:Stylus D78;CLS:PRINTER;"
-         "DES:EPSON Stylus D78;", 1, 'Epson Stylus D68'),
-        ("MFG:Hewlett-Packard;MDL:LaserJet 1200 Series;"
-         "CMD:MLC,PCL,POSTSCRIPT;CLS:PRINTER;", 0, 'HP LaserJet 1200'),
-        ("MFG:Hewlett-Packard;MDL:LaserJet 3390 Series;"
-         "CMD:MLC,PCL,POSTSCRIPT;CLS:PRINTER;", 0, 'HP LaserJet 3390'),
-        ("MFG:Hewlett-Packard;MDL:PSC 2200 Series;CMD:MLC,PCL,PML,DW-PCL,DYN;"
-         "CLS:PRINTER;1284.4DL:4d,4e,1;", 0, "HP PSC 22[01]0"),
-        ("MFG:HEWLETT-PACKARD;MDL:DESKJET 990C;CMD:MLC,PCL,PML;CLS:PRINTER;"
-         "DES:Hewlett-Packard DeskJet 990C;", 0, "HP DeskJet 990C"),
-        ("CLASS:PRINTER;MODEL:HP LaserJet 6MP;MANUFACTURER:Hewlett-Packard;"
-         "DESCRIPTION:Hewlett-Packard LaserJet 6MP Printer;"
-         "COMMAND SET:PJL,MLC,PCLXL,PCL,POSTSCRIPT;", 0, "HP LaserJet (6P/)?6MP"),
-        # Canon PIXMA iP3000 (from gutenprint)
-        ("MFG:Canon;CMD:BJL,BJRaster3,BSCCe;SOJ:TXT01;MDL:iP3000;CLS:PRINTER;"
-         "DES:Canon iP3000;VER:1.09;STA:10;FSI:03;", 1, "Canon PIXMA iP3000"),
-        ("MFG:HP;MDL:Deskjet 5400 series;CMD:MLC,PCL,PML,DW-PCL,DESKJET,DYN;"
-         "1284.4DL:4d,4e,1;CLS:PRINTER;DES:5440;",
-         1, "HP DeskJet (5440|5550)"), # foomatic-db-hpijs used to say 5440
-        ("MFG:Hewlett-Packard;MDL:HP LaserJet 3390;"
-         "CMD:PJL,MLC,PCL,POSTSCRIPT,PCLXL;",
-         0, "HP LaserJet 3390"),
-        # Ricoh printers should use PostScript versions of
-        # manufacturer's PPDs (bug #550315 comment #8).
-        ("MFG:RICOH;MDL:Aficio 3045;",
-         0, "Ricoh Aficio 3045 PS"),
-        # Don't mind which driver gets used here so long as it isn't
-        # gutenprint (bug #645993).
-        ("MFG:Brother;MDL:HL-2030;",
-         0 | FLAG_INVERT | FLAG_IGNORE_STATUS, ".*Gutenprint"),
-        # Make sure we get a colour driver for this one, see launchpad
-        # #669152.
-        ("MFG:Xerox;MDL:6250DP;",
-         1, ".*(Postscript|pcl5e)"),
-
-        # Generic models
-        ("MFG:New;MDL:Unknown PS Printer;CMD:POSTSCRIPT;",
-         2, "Generic postscript printer"),
-        # Make sure pxlcolor is used for PCLXL.  The gutenprint driver
-        # is black and white, and pxlcolor is the foomatic-recommended
-        # generic driver for "Generic PCL 6/PCL XL Printer".
-        ("MFG:New;MDL:Unknown PCL6 Printer;CMD:PCLXL;", 2,
-         "Generic PCL 6.*pxlcolor"),
-        ("MFG:New;MDL:Unknown PCL5e Printer;CMD:PCL5e;", 2, "Generic PCL 5e"),
-        ("MFG:New;MDL:Unknown PCL5c Printer;CMD:PCL5c;", 2, "Generic PCL 5c"),
-        ("MFG:New;MDL:Unknown PCL5 Printer;CMD:PCL5;", 2, "Generic PCL 5"),
-        ("MFG:New;MDL:Unknown PCL3 Printer;CMD:PCL;", 2, "Generic PCL"),
-        ("MFG:New;MDL:Unknown Printer;", 100, None),
-        ]
-
-    if stdin_deviceid:
-        idlist = [(raw_input ('Device ID: '), 2, '')]
-
-    all_passed = True
-    for id, max_status_code, modelre in idlist:
-        flags = max_status_code & ~MASK_STATUS
-        max_status_code &= MASK_STATUS
-        id_dict = parseDeviceID (id)
-        (status, ppdname) = ppds.getPPDNameFromDeviceID (id_dict["MFG"],
-                                                         id_dict["MDL"],
-                                                         id_dict["DES"],
-                                                         id_dict["CMD"])
-        ppddict = ppds.getInfoFromPPDName (ppdname)
-        if flags & FLAG_IGNORE_STATUS:
-            status = max_status_code
-
-        if status < max_status_code:
-            success = True
-        else:
-            if status == max_status_code:
-                match = re.match (modelre,
-                                  _singleton (ppddict['ppd-make-and-model']),
-                                  re.I)
-                success = match != None
-            else:
-                success = False
-
-
-        if flags & FLAG_INVERT:
-            success = not success
-
-        if success:
-            result = "PASS"
-        else:
-            result = "*** FAIL ***"
-
-        print "%s: %s %s (%s)" % (result, id_dict["MFG"], id_dict["MDL"],
-                                  _singleton (ppddict['ppd-make-and-model']))
-        all_passed = all_passed and success
-
-    if not all_passed:
-        raise RuntimeError
+    print ("usage: ppds.py [--deviceid] [--list-models] [--list-ids] [--debug]")
